const path = require('path');

const HtmlWebpackPlugin = require('html-webpack-plugin');
const ReactRefreshWebpackPlugin = require('@pmmmwh/react-refresh-webpack-plugin');

const isProduction = process.env.NODE_ENV === 'production';
const isDevelopment = !isProduction;

const port = process.env.PORT ?? 3001;

const context = path.join(__dirname, 'src');

module.exports = {
  mode: isProduction ? 'production' : 'development',
  bail: isProduction,
  context,
  entry: {
    src: './index.jsx',
  },
  output: {
    path: path.resolve(__dirname, 'dist'),
    publicPath: '/',
    filename: '[name].[chunkhash:8].js',
    assetModuleFilename: '[name].[hash:8][ext]',
  },
  resolve: {
    extensions: ['.js', '.jsx'],
  },
  experiments: {
    topLevelAwait: true,
  },
  module: {
    rules: [
      {
        test: /\.jsx?$/,
        include: [
          path.resolve(context),
        ],
        use: [
          {
            loader: 'babel-loader',
            options: {
              plugins: [isDevelopment && 'react-refresh/babel'].filter(Boolean),
            },
          },
        ],
      },
    ].filter(Boolean),
  },
  plugins: [
    new HtmlWebpackPlugin({
      template: 'index.html',
    }),
    isDevelopment && new ReactRefreshWebpackPlugin(),
  ].filter(Boolean),
  devServer: {
    compress: true,
    historyApiFallback: true,
    hot: isDevelopment,
    port,
    static: {
      directory: path.join(__dirname, 'dist'),
      watch: isDevelopment,
    },
  },
};
